@@ -24,24 +24,11 @@
 
 #include <algorithm>
 
+using std::function;
 using std::vector;
 using std::sort;
 using std::max;
-
-static WavData
-truncate (const WavData& in_data, double seconds)
-{
-  WavData out_data = in_data;
-  vector<float> short_samples = in_data.samples();
-
-  const size_t want_n_samples = lrint (in_data.sample_rate() * seconds) * in_data.n_channels();
-  if (short_samples.size() > want_n_samples)
-    {
-      short_samples.resize (want_n_samples);
-      out_data.set_samples (short_samples);
-    }
-  return out_data;
-}
+using std::min;
 
 static WavData
 get_speed_clip (double location, const WavData& in_data, double clip_seconds)
@@ -158,7 +145,7 @@ public:
     frames_per_block (mark_sync_frame_count() + mark_data_frame_count())
   {
     // constructor is run in the main thread; everything that is not thread-safe must happen here
-    auto sync_finder_bits = SyncFinder::get_sync_bits (key, in_data, SyncFinder::Mode::BLOCK);
+    auto sync_finder_bits = SyncFinder::get_sync_bits (key, SyncFinder::Mode::BLOCK);
     for (size_t bit = 0; bit < sync_finder_bits.size(); bit++)
       {
         for (const auto& frame_bit : sync_finder_bits[bit])
@@ -175,23 +162,31 @@ public:
       }
     std::sort (sync_bits.begin(), sync_bits.end(), [](const auto& s1, const auto& s2) { return s1.frame < s2.frame; });
   }
-  void
-  start_prepare_job (ThreadPool& thread_pool, const SpeedScanParams& scan_params)
+  struct Jobs
   {
-    thread_pool.add_job ([this, &scan_params]() { prepare_mags (scan_params); });
-  }
+    function<void()>         prepare_job;
+    vector<function<void()>> search_jobs;
+    function<void()>         free_memory;
+  };
+  Jobs
+  get_jobs (const SpeedScanParams& scan_params, double speed)
+  {
+    Jobs jobs;
+
+    jobs.prepare_job = [this, &scan_params]() { prepare_mags (scan_params); };
 
-  void
-  start_search_jobs (ThreadPool& thread_pool, const SpeedScanParams& scan_params, double speed)
-  {
     result_scores.clear();
 
     for (int p = -scan_params.n_steps; p <= scan_params.n_steps; p++)
       {
         const double relative_speed = pow (scan_params.step, p) * speed / center;
 
-        thread_pool.add_job ([relative_speed, this]() { compare (relative_speed); });
+        jobs.search_jobs.push_back ([relative_speed, this]() { compare (relative_speed); });
       }
+
+    jobs.free_memory = [this]() { sync_matrix.resize (0, 0); };
+
+    return jobs;
   }
 
   vector<Score>
@@ -209,10 +204,8 @@ public:
 void
 SpeedSync::prepare_mags (const SpeedScanParams& scan_params)
 {
-  WavData in_data_trc (truncate (in_data, scan_params.seconds / center));
-
   // we downsample the audio by factor 2 to improve performance
-  WavData in_data_sub (resample_ratio (in_data_trc, center / 2, Params::mark_sample_rate / 2));
+  WavData in_data_sub (resample_ratio_truncate (in_data, center / 2, Params::mark_sample_rate / 2, /* truncate to length */ scan_params.seconds / center));
 
   const int sub_frame_size = Params::frame_size / 2;
   const int sub_sync_search_step = Params::sync_search_step / 2;
@@ -237,7 +230,9 @@ SpeedSync::prepare_mags (const SpeedScanParams& scan_params)
     {
       int col = 0;
       const std::vector<float>& samples = in_data_sub.samples();
-      vector<float> fft_out_db;
+      std::array<float, Params::max_band - Params::min_band + 1> fft_out_db;
+
+      fft_out_db.fill (0);
 
       for (int ch = 0; ch < in_data_sub.n_channels(); ch++)
         {
@@ -251,7 +246,7 @@ SpeedSync::prepare_mags (const SpeedScanParams& scan_params)
             {
               const float min_db = -96;
 
-              fft_out_db.push_back (db_from_complex (out[i * 2], out[i * 2 + 1], min_db));
+              fft_out_db[i - Params::min_band] += db_from_complex (out[i * 2], out[i * 2 + 1], min_db);
             }
         }
       for (const auto& sync_bit : sync_bits)
@@ -429,7 +424,6 @@ class SpeedSearch
 {
   vector<std::unique_ptr<SpeedSync>> speed_sync;
   SpeedSync      *center_speed_sync = nullptr;
-  ThreadPool&     thread_pool;
   const WavData&  in_data;
   WavData         clipped_in_data;
   double          clip_location;
@@ -444,8 +438,7 @@ class SpeedSearch
     return (*it).get();
   }
 public:
-  SpeedSearch (ThreadPool& thread_pool, const WavData& in_data, double clip_location) :
-    thread_pool (thread_pool),
+  SpeedSearch (const WavData& in_data, double clip_location) :
     in_data (in_data),
     clip_location (clip_location)
   {
@@ -459,15 +452,12 @@ public:
     printf ("range = [ %.2f .. %.2f ]\n", bound (-1), bound (1));
   }
 
-  void start_prepare_jobs (const Key& key, const SpeedScanParams& scan_params, const vector<double>& speeds);
-  void start_search_jobs (const SpeedScanParams& scan_params);
-  void start_refine_jobs (const SpeedScanParams& scan_params, double speed);
+  vector<SpeedSync::Jobs> get_jobs (const Key& key, const SpeedScanParams& scan_params, const vector<double>& speeds);
   vector<SpeedSync::Score> get_results();
-  vector<SpeedSync::Score> get_refine_results();
 };
 
-void
-SpeedSearch::start_prepare_jobs (const Key& key, const SpeedScanParams& scan_params, const vector<double>& speeds)
+vector<SpeedSync::Jobs>
+SpeedSearch::get_jobs (const Key& key, const SpeedScanParams& scan_params, const vector<double>& speeds)
 {
   /* speed is between 0.8 and 1.25, so we use a clip seconds factor of 1.3 to provide enough samples */
   clipped_in_data = get_speed_clip (clip_location, in_data, scan_params.seconds * 1.3);
@@ -484,15 +474,10 @@ SpeedSearch::start_prepare_jobs (const Key& key, const SpeedScanParams& scan_par
         }
     }
 
+  vector<SpeedSync::Jobs> jobs;
   for (auto& s : speed_sync)
-    s->start_prepare_job (thread_pool, scan_params);
-}
-
-void
-SpeedSearch::start_search_jobs (const SpeedScanParams& scan_params)
-{
-  for (auto& s : speed_sync)
-    s->start_search_jobs (thread_pool, scan_params, s->center_speed());
+    jobs.push_back (s->get_jobs (scan_params, s->center_speed()));
+  return jobs;
 }
 
 vector<SpeedSync::Score>
@@ -507,19 +492,6 @@ SpeedSearch::get_results()
   return scores;
 }
 
-void
-SpeedSearch::start_refine_jobs (const SpeedScanParams& scan_params, double speed)
-{
-  center_speed_sync = find_closest_speed_sync (speed);
-  center_speed_sync->start_search_jobs (thread_pool, scan_params, speed);
-}
-
-vector<SpeedSync::Score>
-SpeedSearch::get_refine_results()
-{
-  return center_speed_sync->get_scores();
-}
-
 
 static void
 select_n_best_scores (vector<SpeedSync::Score>& scores, size_t n)
@@ -601,6 +573,53 @@ get_best_clip_location (const Key& key, const WavData& in_data, double seconds,
   return clip_location;
 }
 
+/* Try to process blocks of jobs on our ThreadPool with high concurrency. Examples:
+ *
+ * number of jobs: split_jobs with threads == 32
+* 1: 1
+* 2: 2
+* 3: 3
+* [...]
+* 31: 31
+* 32: 32
+* 33: 17 16
+* 34: 17 17
+* 35: 18 17
+* 36: 18 18
+* [...]
+* 63: 32 31
+* 64: 32 32
+* 65: 32 17 16
+* 66: 32 17 17
+* [...]
+*/
+vector<int>
+split_jobs (int jobs, int threads)
+{
+  vector<int> split_jobs;
+
+  auto update_split_jobs = [&] (int j) {
+    assert (j >= 0 && j <= jobs);
+    if (j)
+      {
+        split_jobs.push_back (j);
+        jobs -= j;
+      }
+  };
+  /* as long as the remaining number of jobs is very large, simply process one block using all threads */
+  while (jobs > 2 * threads)
+    update_split_jobs (threads);
+
+  /* remaining jobs in [threads, 2 * threads]: process half of the remaining jobs */
+  if (jobs > threads)
+    update_split_jobs ((jobs + 1) / 2);
+
+  /* process remaining jobs */
+  update_split_jobs (jobs);
+
+  return split_jobs;
+}
+
 vector<DetectSpeedResult>
 detect_speed (const vector<Key>& key_list, const WavData& in_data, bool print_results)
 {
@@ -668,15 +687,36 @@ detect_speed (const vector<Key>& key_list, const WavData& in_data, bool print_re
   ThreadPool thread_pool;
 
   auto run_search = [&] (const SpeedScanParams& scan_params, auto get_speeds) {
-    for (auto& key_speed_search : key_speed_search_vec)
-      key_speed_search.speed_search->start_prepare_jobs (key_speed_search.key, scan_params, get_speeds (key_speed_search));
-
-    thread_pool.wait_all();
+    vector<SpeedSync::Jobs> jobs;
 
     for (auto& key_speed_search : key_speed_search_vec)
-      key_speed_search.speed_search->start_search_jobs (scan_params);
+      {
+        auto more_jobs = key_speed_search.speed_search->get_jobs (key_speed_search.key, scan_params, get_speeds (key_speed_search));
+        jobs.insert (jobs.end(), more_jobs.begin(), more_jobs.end());
+      }
 
-    thread_pool.wait_all();
+    size_t start = 0;
+    for (size_t count : split_jobs (jobs.size(), thread_pool.n_threads()))
+      {
+        for (size_t i = 0; i < count; i++)
+          thread_pool.add_job (jobs[start + i].prepare_job);
+
+        thread_pool.wait_all();
+
+        for (size_t i = 0; i < count; i++)
+          {
+            for (auto job : jobs[start + i].search_jobs)
+              thread_pool.add_job (job);
+          }
+
+        thread_pool.wait_all();
+
+        for (size_t i = 0; i < count; i++)
+          jobs[start + i].free_memory();
+
+        start += count;
+      }
+    assert (start == jobs.size());
 
     for (auto& key_speed_search : key_speed_search_vec)
       key_speed_search.scores = key_speed_search.speed_search->get_results();
@@ -687,7 +727,7 @@ detect_speed (const vector<Key>& key_list, const WavData& in_data, bool print_re
     {
       const double clip_location = get_best_clip_location (key, in_data, scan1.seconds, clip_candidates);
 
-      key_speed_search_vec.push_back ({key, std::make_unique<SpeedSearch> (thread_pool, in_data, clip_location), {}});
+      key_speed_search_vec.push_back ({key, std::make_unique<SpeedSearch> (in_data, clip_location), {}});
     }
   run_search (scan1, [] (auto& key_speed_search) -> vector<double>
     {
@@ -706,29 +746,15 @@ detect_speed (const vector<Key>& key_list, const WavData& in_data, bool print_re
       return speeds;
     });
 
-  /* improve or refine best match */
+  /* improve best match */
   for (auto& key_speed_search : key_speed_search_vec)
     select_n_best_scores (key_speed_search.scores, 1);
 
-  if (Params::detect_speed_patient)
+  run_search (scan3, [] (auto& key_speed_search) -> vector<double>
     {
-      // slower version: prepare magnitudes again, according to best speed
-      run_search (scan3, [] (auto& key_speed_search) -> vector<double>
-        {
-          return { key_speed_search.scores[0].speed };
-        });
-    }
-  else
-    {
-      // faster version: keep already computed magnitudes
-      for (auto& key_speed_search : key_speed_search_vec)
-        key_speed_search.speed_search->start_refine_jobs (scan3, key_speed_search.scores[0].speed);
+      return { key_speed_search.scores[0].speed };
+    });
 
-      thread_pool.wait_all();
-
-      for (auto& key_speed_search : key_speed_search_vec)
-        key_speed_search.scores = key_speed_search.speed_search->get_refine_results();
-    }
   for (auto& key_speed_search : key_speed_search_vec)
     {
       double best_speed = score_smooth_find_best (key_speed_search.scores, 1 - scan3.step, scan3_smooth_distance);
